@@ -1,78 +1,45 @@
-import random
-from typing import List, Dict, Union, Callable, Optional, Any, Type
+from collections.abc import Callable
+from typing import Any
 
 from flask import request
-from numpy.core.defchararray import isnumeric
-from sqlalchemy import inspect, desc, and_, Column
-from sqlalchemy.exc import IntegrityError, DataError, SQLAlchemyError
-from sqlalchemy.orm import (
-    Query,
-    Session,
-    DeclarativeBase,
-    load_only,
-    object_mapper,
-    object_session,
-    selectinload,
-)
-from sqlalchemy.orm.attributes import flag_modified
+from sqlalchemy import and_, inspect
+from sqlalchemy.exc import DataError, IntegrityError, SQLAlchemyError
+from sqlalchemy.orm import DeclarativeBase, Query, Session, object_session
 from sqlalchemy.orm.exc import UnmappedInstanceError
 
 from flarchitect.core.utils import get_primary_key_info
+from flarchitect.database.inspections import (
+    get_model_columns,
+    get_model_relationships,
+)
 from flarchitect.database.utils import (
-    get_table_and_column,
-    get_models_for_join,
-    get_all_columns_and_hybrids,
+    AGGREGATE_FUNCS,
+    create_aggregate_conditions,
     generate_conditions_from_args,
+    get_all_columns_and_hybrids,
+    get_group_by_fields,
+    get_models_for_join,
+    get_primary_key_filters,
+    get_related_b_query,
     get_select_fields,
-    get_primary_keys,
-    get_primary_key_filters, get_related_b_query,
+    get_table_and_column,
+    parse_column_table_and_operator,
+    validate_table_and_column,
 )
 from flarchitect.exceptions import CustomHTTPException
-from flarchitect.utils.decorators import add_page_totals_and_urls, add_dict_to_query
 from flarchitect.utils.config_helpers import get_config_or_model_meta
+from flarchitect.utils.decorators import add_dict_to_query, add_page_totals_and_urls
+
+__all__ = [
+    "paginate_query",
+    "apply_sorting_to_query",
+    "CrudService",
+    "get_model_columns",
+    "get_model_relationships",
+]
 
 
-def get_model_relationships(
-    model: DeclarativeBase, randomise: bool = True
-) -> List[Type[DeclarativeBase]]:
-    """
-    Extracts relationships from a SQLAlchemy model.
-
-    Args:
-        model (DeclarativeBase): The SQLAlchemy model.
-        randomise (bool, optional): Whether to randomise the order of relationships.
-
-    Returns:
-        List[Type[DeclarativeBase]]: A list of related models.
-    """
-    relationships = [
-        relationship.mapper.class_ for relationship in inspect(model).relationships
-    ]
-    if randomise:
-        random.shuffle(relationships)
-    return relationships
-
-
-def get_model_columns(model: DeclarativeBase, randomise: bool = True) -> List[str]:
-    """
-    Extracts column names from a SQLAlchemy model.
-
-    Args:
-        model (DeclarativeBase): The SQLAlchemy model.
-        randomise (bool, optional): Whether to randomize the order of columns.
-
-    Returns:
-        List[str]: A list of column names.
-    """
-    columns = [x.name for x in list(inspect(model).mapper.columns)]
-    if randomise:
-        random.shuffle(columns)
-    return columns
-
-
-def paginate_query(
-    sql_query: Query, page: int = 0, items_per_page: int | None = None
-) -> Query:
+def paginate_query(sql_query: Query, page: int = 0, items_per_page: int | None = None) -> tuple[Query, int]:
     """Applies pagination to a query.
 
     Args:
@@ -81,10 +48,11 @@ def paginate_query(
         items_per_page (int): Number of items per page.
 
     Returns:
-        Query: Paginated query.
+        tuple[Query, int]:
+            A tuple containing the paginated query and the default pagination size.
     """
 
-    def validate_pagination_params(page: int, items_per_page: int):
+    def validate_pagination_params(page: int, items_per_page: int) -> None:
         """Validate pagination parameters.
 
         Args:
@@ -99,9 +67,7 @@ def paginate_query(
         if not str(items_per_page).isnumeric():
             raise CustomHTTPException(400, "Items per page must be an integer.")
 
-    default_pagination_size = get_config_or_model_meta(
-        "API_PAGINATION_SIZE_DEFAULT", default=20
-    )
+    default_pagination_size = get_config_or_model_meta("API_PAGINATION_SIZE_DEFAULT", default=20)
 
     if items_per_page is None:
         items_per_page = default_pagination_size
@@ -109,16 +75,12 @@ def paginate_query(
     validate_pagination_params(page, items_per_page)
 
     return (
-        sql_query.paginate(
-            page=int(page), per_page=int(items_per_page), error_out=False
-        ),
+        sql_query.paginate(page=int(page), per_page=int(items_per_page), error_out=False),
         default_pagination_size,
     )
 
 
-def apply_sorting_to_query(
-    args_dict: Dict[str, Union[str, int]], query: Query, base_model: Callable
-) -> Query:
+def apply_sorting_to_query(args_dict: dict[str, str | int], query: Query, base_model: Callable) -> Query:
     """Applies order_by conditions to a query.
 
     Args:
@@ -129,7 +91,7 @@ def apply_sorting_to_query(
     Returns:
         Query: Query with applied order_by conditions.
     """
-    order_by = args_dict.get("order_by")
+    order_by = args_dict.get("order_by") or args_dict.get("orderby")
     if not order_by:
         return query
 
@@ -141,10 +103,11 @@ def apply_sorting_to_query(
     for order_key in order_by:
         descending = order_key.startswith("-")
         order_key = order_key.lstrip("-")
-        column = get_table_and_column(order_key, base_model)
+        table_name, column_name = get_table_and_column(order_key, base_model)
+        column_attr = getattr(base_model, column_name, None)
 
-        if column:
-            sorts.append(desc(order_key) if descending else order_key)
+        if column_attr:
+            sorts.append(column_attr.desc() if descending else column_attr)
 
     if sorts:
         query = query.order_by(*sorts)
@@ -163,6 +126,34 @@ class CrudService:
         self.model = model
         self.session = session
 
+    def _process_nested_relationships(self, model: DeclarativeBase, data: dict[str, Any]) -> dict[str, Any]:
+        """Recursively build related model instances from nested dictionaries.
+
+        Args:
+            model: SQLAlchemy model to inspect.
+            data: Payload containing potential nested relationship data.
+
+        Returns:
+            dict[str, Any]: Payload with nested dictionaries replaced by model instances.
+        """
+        for relationship in inspect(model).relationships:
+            key = relationship.key
+            if key not in data or data[key] is None:
+                continue
+
+            related_model = relationship.mapper.class_
+            value = data[key]
+
+            if relationship.uselist and isinstance(value, list):
+                data[key] = [related_model(**self._process_nested_relationships(related_model, item)) for item in value]
+            elif not relationship.uselist and isinstance(value, dict):
+                data[key] = related_model(**self._process_nested_relationships(related_model, value))
+
+            for col in relationship.local_columns:
+                data.pop(col.name, None)
+
+        return data
+
     def fetch_related_model_by_name(self, field_name: str) -> Callable:
         """Gets a related model by field name.
 
@@ -186,35 +177,58 @@ class CrudService:
 
         return related_model.mapper.class_
 
-    def filter_query_from_args(
-        self, args_dict: Dict[str, Union[str, int]], query=None
-    ) -> Query:
-        """Filters a query based on request arguments.
+    def filter_query_from_args(self, args_dict: dict[str, str | int], query=None) -> Query:
+        """Build a query applying joins, filters, grouping and aggregation.
 
         Args:
-            args_dict (Dict[str, Union[str, int]]): Dictionary containing filtering, sorting, pagination, and aggregation conditions.
+            args_dict: Dictionary containing query parameters.
+            query: Optional existing SQLAlchemy query to build upon.
 
         Returns:
-            Query: The filtered query.
+            Query with all requested transformations applied.
         """
-        join_models = get_models_for_join(args_dict, self.fetch_related_model_by_name)
+
+        allow_join = get_config_or_model_meta("API_ALLOW_JOIN", model=self.model, default=False)
+        join_models = get_models_for_join(args_dict, self.fetch_related_model_by_name) if allow_join else {}
+
         all_columns, all_models = get_all_columns_and_hybrids(self.model, join_models)
 
-        conditions = [
-            condition
-            for condition in generate_conditions_from_args(
-                args_dict, self.model, all_columns, all_models, join_models
-            )
-            if condition is not None
-        ]
+        conditions = [condition for condition in generate_conditions_from_args(args_dict, self.model, all_columns, all_models, join_models) if condition is not None]
 
-        query = self.initialize_query(args_dict, all_columns, query)
+        allow_select = get_config_or_model_meta("API_ALLOW_SELECT_FIELDS", model=self.model, default=True)
+        select_fields = get_select_fields(args_dict, self.model, all_columns) if allow_select else []
 
-        if conditions and get_config_or_model_meta(
-            "API_ALLOW_FILTER", model=self.model, default=True
-        ):
+        allow_group = get_config_or_model_meta("API_ALLOW_GROUPBY", model=self.model, default=False)
+        group_by_fields = get_group_by_fields(args_dict, all_columns, self.model) if allow_group else []
+
+        allow_agg = get_config_or_model_meta("API_ALLOW_AGGREGATION", model=self.model, default=False)
+        aggregate_conditions = create_aggregate_conditions(args_dict) if allow_agg else {}
+        agg_fields = []
+        for key, label in aggregate_conditions.items():
+            column_name, table_name, func_name = parse_column_table_and_operator(key, self.model)
+            model_column, _ = validate_table_and_column(table_name, column_name, all_columns)
+            agg_func = AGGREGATE_FUNCS.get(func_name)
+            if agg_func:
+                agg_label = label or f"{column_name}_{func_name}"
+                agg_fields.append(agg_func(model_column).label(agg_label))
+
+        query = query or self.session.query(self.model)
+
+        if allow_join and join_models:
+            for mdl in join_models.values():
+                query = query.join(mdl)
+
+        if agg_fields or group_by_fields:
+            query = query.with_entities(*(group_by_fields + agg_fields))
+        elif select_fields:
+            query = query.with_entities(*select_fields)
+
+        if conditions and get_config_or_model_meta("API_ALLOW_FILTER", model=self.model, default=True):
             query = query.filter(and_(*conditions))
 
+        if group_by_fields:
+            query = query.group_by(*group_by_fields)
+
         return query
 
     def order_query(self, args_dict: dict, query: Query) -> Query:
@@ -227,45 +241,10 @@ class CrudService:
         Returns:
 
         """
-        if get_config_or_model_meta(
-            "API_ALLOW_ORDER_BY", model=self.model, default=True
-        ):
+        if get_config_or_model_meta("API_ALLOW_ORDER_BY", model=self.model, default=True):
             query = apply_sorting_to_query(args_dict, query, self.model)
         return query
 
-    def initialize_query(
-        self,
-        args_dict: Dict[str, Union[str, int]],
-        all_columns: Dict[str, Column],
-        query=None,
-    ) -> Query:
-        """Helper function to initialize the query based on select fields.
-
-        Args:
-            args_dict (Dict[str, Union[str, int]]): Dictionary containing filtering, sorting, pagination, and aggregation conditions.
-            all_columns (Dict[str, Column]): Dictionary of all columns and hybrids.
-
-        Returns:
-            Query: Initialized query.
-        """
-        allow_select = get_config_or_model_meta(
-            "API_ALLOW_SELECT_FIELDS", model=self.model, default=True
-        )
-        select_fields = (
-            get_select_fields(args_dict, self.model, all_columns)
-            if allow_select
-            else []
-        )
-
-        if select_fields:
-            if query:
-                return query.select_from(*select_fields)
-            return self.session.query(*select_fields)
-
-        if query:
-            return query
-        return self.session.query(self.model)
-
     def apply_soft_delete_filter(self, query: Query) -> Query:
         """Adds a soft delete filter to the query if applicable.
 
@@ -279,19 +258,11 @@ class CrudService:
             return query
 
         show_deleted = request.args.get("include_deleted", None)
-        deleted_attr = get_config_or_model_meta(
-            "API_SOFT_DELETE_ATTRIBUTE", default=None
-        )
-        soft_delete_values = get_config_or_model_meta(
-            "API_SOFT_DELETE_VALUES", default=False
-        )
+        deleted_attr = get_config_or_model_meta("API_SOFT_DELETE_ATTRIBUTE", default=None)
+        soft_delete_values = get_config_or_model_meta("API_SOFT_DELETE_VALUES", default=False)
 
         if not show_deleted and deleted_attr:
-            models = {
-                getattr(inspect(desc["entity"]).mapper, "class_", None)
-                for desc in query.column_descriptions
-                if desc["entity"]
-            }
+            models = {getattr(inspect(desc["entity"]).mapper, "class_", None) for desc in query.column_descriptions if desc["entity"]}
 
             for model in models:
                 if hasattr(model, deleted_attr):
@@ -304,13 +275,13 @@ class CrudService:
     @add_dict_to_query
     def get_query(
         self,
-        args_dict: Dict[str, Union[str, int]],
-        lookup_val: Optional[Union[int, str]] = None,
-        alt_field: Optional[str] = None,
+        args_dict: dict[str, str | int],
+        lookup_val: int | str | None = None,
+        alt_field: str | None = None,
         many: bool = True,
         other_model=None,
         **kwargs,
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         """Gets the query result after applying filtering, sorting, pagination, and aggregation.
 
         Args:
@@ -331,13 +302,18 @@ class CrudService:
 
             else:
                 query = self.session.query(base_model)
+                callback = get_config_or_model_meta("API_FILTER_CALLBACK", model=base_model, default=None)
+                if callback:
+                    query = callback(query, self.model, args_dict)
 
-            if alt_field:
-                query = query.filter(getattr(base_model, alt_field) == lookup_val)
-            else:
-                query = query.filter_by(
-                    **get_primary_key_filters(base_model, lookup_val)
-                )
+            query = query.filter(getattr(base_model, alt_field) == lookup_val) if alt_field else query.filter_by(**get_primary_key_filters(base_model, lookup_val))
+
+            if get_config_or_model_meta("API_SOFT_DELETE", model=base_model, default=False):
+                show_deleted = request.args.get("include_deleted", None)
+                deleted_attr = get_config_or_model_meta("API_SOFT_DELETE_ATTRIBUTE", model=base_model, default=None)
+                soft_delete_values = get_config_or_model_meta("API_SOFT_DELETE_VALUES", model=base_model, default=None)
+                if not show_deleted and deleted_attr:
+                    query = query.filter(getattr(base_model, deleted_attr) == soft_delete_values[0])
 
             result = query.one_or_none()
 
@@ -354,43 +330,35 @@ class CrudService:
             query = get_related_b_query(kwargs.get("join_model"), self.model, lookup_val, self.session)
 
             # relationships i.e /authors/1/books was returning a 200 when author is None. This fixes it.
-            if (
-                query.count()== 0
-            ):
-                raise CustomHTTPException(
-                    404, f"{kwargs.get('join_model').__name__} not found."
-                )
+            if query.count() == 0:
+                raise CustomHTTPException(404, f"{kwargs.get('join_model').__name__} not found.")
 
             query = self.filter_query_from_args(args_dict, query)
         else:
             query = self.filter_query_from_args(args_dict)
 
-        count = query.count()
+        callback = get_config_or_model_meta("API_FILTER_CALLBACK", model=base_model, default=None)
+        if callback:
+            query = callback(query, self.model, args_dict)
 
+        count = query.count()
         order_query = self.order_query(args_dict, query)
 
-        paginated_query, default_pagination_size = paginate_query(
-            order_query, args_dict.get("page", 1), args_dict.get("limit")
-        )
+        # Apply soft delete filtering before pagination to avoid operating on
+        # paginated objects, which lack SQLAlchemy query attributes such as
+        # ``column_descriptions``.
+        filtered_query = self.apply_soft_delete_filter(order_query)
 
-        final_query = self.apply_soft_delete_filter(paginated_query)
+        paginated_query, default_pagination_size = paginate_query(filtered_query, args_dict.get("page", 1), args_dict.get("limit"))
 
         return {
-            "query": (
-                final_query.all()
-                if hasattr(final_query, "all")
-                else final_query.items if hasattr(final_query, "items") else final_query
-            ),
-            "limit": (
-                int(args_dict.get("limit"))
-                if args_dict.get("limit")
-                else default_pagination_size
-            ),
+            "query": (paginated_query.all() if hasattr(paginated_query, "all") else paginated_query.items if hasattr(paginated_query, "items") else paginated_query),
+            "limit": (int(args_dict.get("limit")) if args_dict.get("limit") else default_pagination_size),
             "page": int(args_dict.get("page")) if args_dict.get("page") else 1,
             "total_count": count,
         }
 
-    def add_object(self, data_dict: Dict[str, Any], *args, **kwargs) -> Callable:
+    def add_object(self, data_dict: dict[str, Any], *args, **kwargs) -> Callable:
         """Adds a new object to the database.
 
         Args:
@@ -404,17 +372,22 @@ class CrudService:
             DataError: If there is a data type error.
         """
         try:
-            obj = self.model(**data_dict)
+            allow_nested = get_config_or_model_meta("ALLOW_NESTED_WRITES", model=self.model, default=False)
+            payload = self._process_nested_relationships(self.model, data_dict.copy()) if allow_nested else data_dict
+            obj = self.model(**payload)
+
+            callback = get_config_or_model_meta("API_ADD_CALLBACK", model=self.model, default=None)
+            if callback:
+                obj = callback(obj, self.model)
+
             self.session.add(obj)
             self.session.commit()
             return obj
         except (IntegrityError, DataError) as e:
             self.session.rollback()
-            raise CustomHTTPException(422, str(e.orig))
+            raise CustomHTTPException(422, str(e.orig)) from e
 
-    def update_object(
-        self, lookup_val: Union[int, str], data_dict: Dict[str, Any], *args, **kwargs
-    ) -> Callable:
+    def update_object(self, lookup_val: int | str, data_dict: dict[str, Any], *args, **kwargs) -> Callable:
         """Updates an existing object in the database.
 
         Args:
@@ -429,24 +402,28 @@ class CrudService:
             DataError: If there is a data type error.
         """
         try:
-            obj = (
-                self.session.query(self.model)
-                .filter_by(**get_primary_key_filters(self.model, lookup_val))
-                .one_or_none()
-            )
+            obj = self.session.query(self.model).filter_by(**get_primary_key_filters(self.model, lookup_val)).one_or_none()
             if obj is None:
                 raise CustomHTTPException(404, f"{self.model.__name__} not found.")
-            for key, value in data_dict.items():
+
+            allow_nested = get_config_or_model_meta("ALLOW_NESTED_WRITES", model=self.model, default=False)
+            update_payload = self._process_nested_relationships(self.model, data_dict.copy()) if allow_nested else data_dict
+            for key, value in update_payload.items():
                 setattr(obj, key, value)
+
+            callback = get_config_or_model_meta("API_UPDATE_CALLBACK", model=self.model, default=None)
+            if callback:
+                obj = callback(obj, self.model)
+
             self.session.commit()
             return obj
         except (IntegrityError, DataError) as e:
             self.session.rollback()
-            raise CustomHTTPException(422, str(e.orig))
+            raise CustomHTTPException(422, str(e.orig)) from e
 
-    def delete_object(self, lookup_val: Union[int, str], *args, **kwargs) -> None:
+    def delete_object(self, lookup_val: int | str, *args, **kwargs) -> None:
         """
-        Deletes an object from the database, along with related and dependent models based on cascade_delete flag.
+        Deletes an object from the database.
 
         Args:
             lookup_val (Union[int, str]): Value to lookup the object by primary key.
@@ -457,45 +434,49 @@ class CrudService:
         # Fetch cascade_delete flag from request args
         cascade_delete = int(request.args.get("cascade_delete", 0)) == 1
 
-        obj = (
-            self.session.query(self.model)
-            .filter_by(**get_primary_key_filters(self.model, lookup_val))
-            .one_or_none()
-        )
+        obj = self.session.query(self.model).filter_by(**get_primary_key_filters(self.model, lookup_val)).one_or_none()
 
         if obj is None:
             raise CustomHTTPException(404, f"{self.model.__name__} not found.")
 
+        callback = get_config_or_model_meta("API_REMOVE_CALLBACK", model=self.model, default=None)
+        if callback:
+            obj = callback(obj, self.model)
+
+        # Handle soft deletes when configured.
+        if get_config_or_model_meta("API_SOFT_DELETE", model=self.model, default=False) and not cascade_delete:
+            deleted_attr = get_config_or_model_meta("API_SOFT_DELETE_ATTRIBUTE", model=self.model, default=None)
+            soft_delete_values = get_config_or_model_meta("API_SOFT_DELETE_VALUES", model=self.model, default=None)
+
+            if not deleted_attr or not soft_delete_values:
+                raise CustomHTTPException(500, "Soft delete misconfigured")
+
+            setattr(obj, deleted_attr, soft_delete_values[1])
+            self.session.commit()
+            return None, 200
+
         with self.session.no_autoflush:
             self.session.delete(obj)
             try:
-
-                if (
-                    not get_config_or_model_meta(
-                        "API_ALLOW_CASCADE_DELETE", model=self.model, default=True
-                    )
-                    or request.args.get("cascade_delete") != "1"
-                ):
+                if not get_config_or_model_meta("API_ALLOW_CASCADE_DELETE", model=self.model, default=True) or request.args.get("cascade_delete") != "1":
                     self.session.commit()
-                    return 204
+                    return None, 200
 
                 # Perform recursive delete based on cascade_delete flag
 
-                objects_touched = recursive_delete(obj, cascade_delete)
+                recursive_delete(obj, cascade_delete)
                 self.session.commit()
 
             except SQLAlchemyError as e:
                 self.session.rollback()
-                if get_config_or_model_meta(
-                    "API_ALLOW_CASCADE_DELETE", model=self.model, default=False
-                ):
-                    error_msg = f"Error deleting object, use url parameter `cascade_delete=1` to attempt cascade delete"
+                if get_config_or_model_meta("API_ALLOW_CASCADE_DELETE", model=self.model, default=False):
+                    error_msg = "Error deleting object, use url parameter `cascade_delete=1` to attempt cascade delete"
                 else:
-                    error_msg = f"Error deleting object"
+                    error_msg = "Error deleting object"
 
-                raise CustomHTTPException(409, error_msg)
+                raise CustomHTTPException(409, error_msg) from e
 
-        return 204
+        return None, 200
 
 
 def recursive_delete(obj, cascade_delete=True, visited=None, objects_touched=None, parent=None):
@@ -509,6 +490,7 @@ def recursive_delete(obj, cascade_delete=True, visited=None, objects_touched=Non
         objects_touched (list): List of objects touched (deleted) during the recursive process.
         parent: The parent object of the current recursion step, used to prevent backward traversal in cyclic relationships.
     """
+
     def get_obj_id(obj):
         mapper = inspect(obj.__class__)
         return (
