@@ -1,19 +1,28 @@
-import inspect
+from collections.abc import Callable
+from contextlib import suppress
 from datetime import datetime
-from typing import List, Tuple, Dict, Callable, Any, Union, Optional, Type
+from typing import Any
 
-from sqlalchemy import func, Column, or_, Integer, Float, Date, Boolean, inspect
+from sqlalchemy import Boolean, Column, Date, Float, Integer, func, inspect, or_
 from sqlalchemy.exc import StatementError
 from sqlalchemy.ext.hybrid import hybrid_property
-from sqlalchemy.orm import class_mapper, DeclarativeBase, InstrumentedAttribute, RelationshipProperty
+from sqlalchemy.orm import (
+    DeclarativeBase,
+    InstrumentedAttribute,
+    RelationshipProperty,
+    class_mapper,
+)
 
 from flarchitect.exceptions import CustomHTTPException
 from flarchitect.logging import logger
-from flarchitect.utils.core_utils import convert_camel_to_snake, convert_kebab_to_snake, convert_case
 from flarchitect.utils.config_helpers import get_config_or_model_meta
+from flarchitect.utils.core_utils import (
+    convert_camel_to_snake,
+    convert_case,
+    convert_kebab_to_snake,
+)
 
-
-OPERATORS: Dict[str, Callable[[Any, Any], Any]] = {
+OPERATORS: dict[str, Callable[[Any, Any], Any]] = {
     "lt": lambda f, a: f < a,
     "le": lambda f, a: f <= a,
     "gt": lambda f, a: f > a,
@@ -36,8 +45,7 @@ AGGREGATE_FUNCS = {
 OTHER_FUNCTIONS = ["groupby", "fields", "join", "orderby"]
 
 
-
-def fetch_related_classes_and_attributes(model: object) -> List[Tuple[str, str]]:
+def fetch_related_classes_and_attributes(model: object) -> list[tuple[str, str]]:
     """Retrieve related class names and attributes for a given SQLAlchemy model.
 
     Args:
@@ -47,14 +55,10 @@ def fetch_related_classes_and_attributes(model: object) -> List[Tuple[str, str]]
         List[Tuple[str, str]]: A list of tuples containing the relationship's
                                attribute name on the model and the class name of the related model.
     """
-    return [
-        (relation.key, relation.mapper.class_.__name__)
-        for relation in class_mapper(model).relationships
-    ]
+    return [(relation.key, relation.mapper.class_.__name__) for relation in class_mapper(model).relationships]
 
-def get_all_columns_and_hybrids(
-    model: DeclarativeBase, join_models: Dict[str, DeclarativeBase]
-) -> Tuple[Dict[str, Dict[str, Union[hybrid_property, InstrumentedAttribute]]], List[DeclarativeBase]]:
+
+def get_all_columns_and_hybrids(model: DeclarativeBase, join_models: dict[str, DeclarativeBase]) -> tuple[dict[str, dict[str, hybrid_property | InstrumentedAttribute]], list[DeclarativeBase]]:
     """
     Retrieves all columns and hybrid properties from the base model and any join models.
 
@@ -68,12 +72,8 @@ def get_all_columns_and_hybrids(
     """
     from flarchitect.utils.general import get_config_or_model_meta
 
-    ignore_underscore = get_config_or_model_meta(
-        key="API_IGNORE_UNDERSCORE_ATTRIBUTES", model=model, default=True
-    )
-    schema_case = get_config_or_model_meta(
-        key="API_SCHEMA_CASE", model=model, default="camel"
-    )
+    ignore_underscore = get_config_or_model_meta(key="API_IGNORE_UNDERSCORE_ATTRIBUTES", model=model, default=True)
+    schema_case = get_config_or_model_meta(key="API_SCHEMA_CASE", model=model, default="camel")
 
     all_columns = {}
     all_models = [model] + list(join_models.values())
@@ -81,14 +81,12 @@ def get_all_columns_and_hybrids(
     for mdl in all_models:
         table_name = convert_case(mdl.__name__, schema_case)
         all_columns[table_name] = {
-            attr: column
-            for attr, column in mdl.__dict__.items()
-            if isinstance(column, (hybrid_property, InstrumentedAttribute))
-            and (not ignore_underscore or not attr.startswith("_"))
+            attr: column for attr, column in mdl.__dict__.items() if isinstance(column, hybrid_property | InstrumentedAttribute) and (not ignore_underscore or not attr.startswith("_"))
         }
 
     return all_columns, all_models
 
+
 def create_pagination_defaults():
     PAGINATION_DEFAULTS = {
         "page": 1,
@@ -100,7 +98,8 @@ def create_pagination_defaults():
 
     return PAGINATION_DEFAULTS, PAGINATION_MAX
 
-def extract_pagination_params(args_dict: Dict[str, str]) -> Tuple[int, int]:
+
+def extract_pagination_params(args_dict: dict[str, str]) -> tuple[int, int]:
     """
     Get the pagination from the request arguments.
 
@@ -110,7 +109,6 @@ def extract_pagination_params(args_dict: Dict[str, str]) -> Tuple[int, int]:
     Returns:
         Tuple[int, int]: Tuple of page and limit.
     """
-    from flarchitect.utils.general import get_config_or_model_meta
 
     # Pagination defaults and maximums
 
@@ -121,58 +119,53 @@ def extract_pagination_params(args_dict: Dict[str, str]) -> Tuple[int, int]:
     limit = int(args_dict.get("limit", PAGINATION_DEFAULTS["limit"]))
 
     if limit > PAGINATION_MAX["limit"]:
-        raise CustomHTTPException(
-            400, f"Limit exceeds maximum value of {PAGINATION_MAX['limit']}"
-        )
+        raise CustomHTTPException(400, f"Limit exceeds maximum value of {PAGINATION_MAX['limit']}")
 
     return page, limit
 
 
 def get_group_by_fields(
-    args_dict: Dict[str, str],
-    all_columns: Dict[str, Dict[str, Union[hybrid_property, InstrumentedAttribute]]],
+    args_dict: dict[str, str],
+    all_columns: dict[str, dict[str, hybrid_property | InstrumentedAttribute]],
     base_model: DeclarativeBase,
-) -> List[Callable]:
-    """
-    Get the group by fields from the request arguments.
+) -> list[Callable]:
+    """Retrieve ``GROUP BY`` columns from request arguments.
 
     Args:
-        args_dict (Dict[str, str]): Dictionary of request arguments.
-        all_columns (Dict[str, Dict[str, Column]]): Nested dictionary of table names and their columns.
-        base_model (DeclarativeBase): The base SQLAlchemy model.
+        args_dict: Dictionary of request arguments.
+        all_columns: Nested dictionary of table names and their columns.
+        base_model: The base SQLAlchemy model.
 
     Returns:
-        List[Callable]: List of conditions to apply in the query.
+        List of columns to apply in the ``GROUP BY`` clause.
     """
-    group_by_fields = []
+
+    group_by_fields: list[Callable] = []
     if "groupby" in args_dict:
         fields = args_dict.get("groupby").split(",")
         for field in fields:
             table_name, column_name = get_table_and_column(field, base_model)
-            model_column, _ = validate_table_and_column(
-                table_name, column_name, all_columns
-            )
+            model_column, _ = validate_table_and_column(table_name, column_name, all_columns)
             group_by_fields.append(model_column)
 
     return group_by_fields
 
 
-def get_join_models(
-    args_dict: Dict[str, str], get_model_func: Callable[[str], DeclarativeBase]
-) -> Dict[str, DeclarativeBase]:
-    """
-    Builds a list of SQLAlchemy models to join based on request arguments.
+def get_models_for_join(args_dict: dict[str, str], get_model_func: Callable[[str], DeclarativeBase]) -> dict[str, DeclarativeBase]:
+    """Build a mapping of models to join based on the ``join`` query parameter.
 
     Args:
-        args_dict (Dict[str, str]): Dictionary of request arguments.
-        get_model_func (Callable): Function to get a model by name.
+        args_dict: Dictionary of request arguments.
+        get_model_func: Function used to resolve a model by name.
 
     Returns:
-        Dict[str, DeclarativeBase]: A dictionary of SQLAlchemy models to join.
+        Dictionary mapping requested join names to SQLAlchemy model classes.
     """
-    models = {}
-    if "join" in args_dict:
-        for join in args_dict["join"].split(","):
+
+    models: dict[str, DeclarativeBase] = {}
+    join_value = args_dict.get("join") or args_dict.get("join_models")
+    if join_value:
+        for join in join_value.split(","):
             model = get_model_func(join)
             if not model:
                 raise CustomHTTPException(400, f"Invalid join model: {join}")
@@ -181,9 +174,7 @@ def get_join_models(
     return models
 
 
-def get_table_column(
-    key: str, all_columns: Dict[str, Dict[str, Any]]
-) -> Tuple[str, str, str]:
+def get_table_column(key: str, all_columns: dict[str, dict[str, Any]]) -> tuple[str, str, str]:
     """
     Get the fully qualified column name (i.e., with table name).
 
@@ -209,10 +200,10 @@ def get_table_column(
 
 
 def get_select_fields(
-    args_dict: Dict[str, str],
+    args_dict: dict[str, str],
     base_model: DeclarativeBase,
-    all_columns: Dict[str, Dict[str, Column]],
-) -> List[Callable]:
+    all_columns: dict[str, dict[str, Column]],
+) -> list[Callable]:
     """
     Get the select fields from the request arguments.
 
@@ -229,15 +220,13 @@ def get_select_fields(
         fields = args_dict.get("fields").split(",")
         for field in fields:
             table_name, column_name = get_table_and_column(field, base_model)
-            model_column, _ = validate_table_and_column(
-                table_name, column_name, all_columns
-            )
+            model_column, _ = validate_table_and_column(table_name, column_name, all_columns)
             select_fields.append(model_column)
 
     return select_fields
 
 
-def parse_or_condition_keys_and_values(key: str, val: str) -> Tuple[List[str], List[str]]:
+def parse_or_condition_keys_and_values(key: str, val: str) -> tuple[list[str], list[str]]:
     """
     Get the 'or' values and keys.
 
@@ -251,23 +240,21 @@ def parse_or_condition_keys_and_values(key: str, val: str) -> Tuple[List[str], L
     # Extract the initial key, remove 'or[' and strip any whitespace
     keys = []
     values = []
-    for val in (key + "=" + val)[3:-1].split(","):
-        key, val = val.split("=")
-        keys.append(key.strip())
-        values.append(val.strip())
+    for item in (key + "=" + val)[3:-1].split(","):
+        key_part, value_part = item.split("=")
+        keys.append(key_part.strip())
+        values.append(value_part.strip())
 
     return keys, values
 
 
-
-
 def generate_conditions_from_args(
-    args_dict: Dict[str, str],
+    args_dict: dict[str, str],
     base_model: DeclarativeBase,
-    all_columns: Dict[str, Dict[str, Column]],
-    all_models: List[DeclarativeBase],
-    join_models: Dict[str, DeclarativeBase],
-) -> List[Callable]:
+    all_columns: dict[str, dict[str, Column]],
+    all_models: list[DeclarativeBase],
+    join_models: dict[str, DeclarativeBase],
+) -> list[Callable]:
     """
     Create filter conditions based on request arguments and model's columns.
 
@@ -289,26 +276,24 @@ def generate_conditions_from_args(
 
     PAGINATION_DEFAULTS, PAGINATION_MAX = create_pagination_defaults()
 
-    for key, value in args_dict.items():
-        if (
-            any(op in key for op in OPERATORS.keys())
-            and not any(func in key for func in [*PAGINATION_DEFAULTS, *OTHER_FUNCTIONS])
-        ):
+    for key, _value in args_dict.items():
+        if any(op in key for op in OPERATORS) and not any(func in key for func in [*PAGINATION_DEFAULTS, *OTHER_FUNCTIONS]):
             if key.startswith("or["):
-                or_keys, or_vals = parse_or_condition_keys_and_values(key, value)
+                or_keys, or_vals = parse_or_condition_keys_and_values(key, _value)
                 or_conditions.extend(
                     create_condition(
-                        *get_table_column(or_key, all_columns), or_val, all_columns, base_model
+                        *get_table_column(or_key, all_columns),
+                        or_val,
+                        all_columns,
+                        base_model,
                     )
-                    for or_key, or_val in zip(or_keys, or_vals)
+                    for or_key, or_val in zip(or_keys, or_vals, strict=False)
                 )
                 continue
 
             table, column, operator = get_table_column(key, all_columns)
             if operator:
-                condition = create_condition(
-                    table, column, operator, value, all_columns, base_model
-                )
+                condition = create_condition(table, column, operator, _value, all_columns, base_model)
                 if condition is not None:
                     conditions.append(condition)
 
@@ -318,27 +303,6 @@ def generate_conditions_from_args(
     return conditions
 
 
-def get_models_for_join(
-    args_dict: Dict[str, str], get_model_func: Callable[[str], DeclarativeBase]
-) -> Dict[str, DeclarativeBase]:
-    """
-    Builds a list of SQLAlchemy models to join based on request arguments.
-
-    Args:
-        args_dict (Dict[str, str]): Dictionary of request arguments.
-        get_model_func (Callable): Function to get a model by name.
-
-    Returns:
-        Dict[str, DeclarativeBase]: A dictionary of SQLAlchemy models to join.
-    """
-    models = {}
-    if "join" in args_dict:
-        for join in args_dict["join"].split(","):
-            model = get_model_func(join)
-            models[join] = model
-
-    return models
-
 def parse_key_and_label(key):
     """
         Get the key and label from the key
@@ -365,9 +329,10 @@ def parse_key_and_label(key):
 
         return key, label
 
+
 def create_aggregate_conditions(
-    args_dict: Dict[str, str]
-) -> Optional[Dict[str, Optional[str]]]:
+    args_dict: dict[str, str],
+) -> dict[str, str | None] | None:
     """
     Creates aggregate conditions based on request arguments and the model's columns.
 
@@ -379,8 +344,8 @@ def create_aggregate_conditions(
     """
     aggregate_conditions = {}
 
-    for key, value in args_dict.items():
-        for func_name in AGGREGATE_FUNCS.keys():
+    for key, _value in args_dict.items():
+        for func_name in AGGREGATE_FUNCS:
             if f"__{func_name}" in key:
                 key, label = parse_key_and_label(key)
                 aggregate_conditions[key] = label
@@ -388,7 +353,7 @@ def create_aggregate_conditions(
     return aggregate_conditions
 
 
-def get_table_and_column(value: str, main_model: DeclarativeBase) -> Tuple[str, str]:
+def get_table_and_column(value: str, main_model: DeclarativeBase) -> tuple[str, str]:
     """
     Get the table and column name from the value.
 
@@ -404,16 +369,12 @@ def get_table_and_column(value: str, main_model: DeclarativeBase) -> Tuple[str,
 
     from flarchitect.utils.general import get_config_or_model_meta
 
-    schema_case = get_config_or_model_meta(
-        "API_SCHEMA_CASE", model=main_model, default="camel"
-    )
+    schema_case = get_config_or_model_meta("API_SCHEMA_CASE", model=main_model, default="camel")
     table_name = convert_case(main_model.__name__, schema_case)
     return table_name, value
 
 
-def parse_column_table_and_operator(
-    key: str, main_model: DeclarativeBase
-) -> Tuple[str, str, str]:
+def parse_column_table_and_operator(key: str, main_model: DeclarativeBase) -> tuple[str, str, str]:
     """
     Get the column and table name from the key.
 
@@ -429,9 +390,7 @@ def parse_column_table_and_operator(
     return column_name, table_name, operator_str
 
 
-def validate_table_and_column(
-    table_name: str, column_name: str, all_columns: Dict[str, Dict[str, Column]]
-) -> Tuple[Column, str]:
+def validate_table_and_column(table_name: str, column_name: str, all_columns: dict[str, dict[str, Column]]) -> tuple[Column, str]:
     """
     Get the column from the column dictionary.
 
@@ -464,9 +423,9 @@ def create_condition(
     column_name: str,
     operator: str,
     value: str,
-    all_columns: Dict[str, Dict[str, Column]],
+    all_columns: dict[str, dict[str, Column]],
     model: DeclarativeBase,
-) -> Optional[Callable]:
+) -> Callable | None:
     """
     Converts a key-value pair from request arguments to a condition.
 
@@ -483,10 +442,7 @@ def create_condition(
     """
     model_column, _ = validate_table_and_column(table_name, column_name, all_columns)
 
-    if type(model_column) is hybrid_property:
-        column_type = get_type_hint_from_hybrid(model_column)
-    else:
-        column_type = model_column.type
+    column_type = get_type_hint_from_hybrid(model_column) if isinstance(model_column, hybrid_property) else model_column.type
 
     if "in" in operator:
         value = value.strip("()").split(",")
@@ -494,10 +450,8 @@ def create_condition(
     if "like" in operator:
         value = f"%{value}%"
 
-    try:
+    with suppress(ValueError):
         value = convert_value_to_type(value, column_type)
-    except ValueError:
-        pass
 
     operator_func = OPERATORS.get(operator)
     if operator_func is None:
@@ -524,7 +478,7 @@ def is_hybrid_property(prop: Any) -> bool:
     return isinstance(prop, hybrid_property)
 
 
-def get_type_hint_from_hybrid(func: Callable) -> Optional[Type]:
+def get_type_hint_from_hybrid(func: Callable) -> type | None:
     """
     Converts a function (hybrid_property) into its returning type.
 
@@ -537,9 +491,7 @@ def get_type_hint_from_hybrid(func: Callable) -> Optional[Type]:
     return func.__annotations__.get("return")
 
 
-def convert_value_to_type(
-    value: Union[str, List[str]], column_type: Any, is_hybrid: bool = False
-) -> Any:
+def convert_value_to_type(value: str | list[str], column_type: Any, is_hybrid: bool = False) -> Any:
     """
     Convert the given string value or list of string values to its appropriate type based on the provided column_type.
 
@@ -551,6 +503,7 @@ def convert_value_to_type(
     Returns:
         Any: The converted value(s).
     """
+
     def convert_to_boolean(val: str) -> bool:
         val = val.lower()
         if val in ["true", "1", "yes", "y"]:
@@ -570,12 +523,12 @@ def convert_value_to_type(
             return convert_to_boolean(val)
         return val
 
-    if isinstance(value, (list, set, tuple)):
+    if isinstance(value, list | set | tuple):
         return [convert_single_value(str(v), column_type) for v in value]
     return convert_single_value(value, column_type)
 
 
-def find_matching_relations(model1: Callable, model2: Callable) -> List[Tuple[str, str]]:
+def find_matching_relations(model1: Callable, model2: Callable) -> list[tuple[str, str]]:
     """Find matching relation fields between two SQLAlchemy models.
 
     Args:
@@ -599,7 +552,9 @@ def find_matching_relations(model1: Callable, model2: Callable) -> List[Tuple[st
     return matching_relations
 
 
-def _get_relation_use_list_and_type(relationship_property: RelationshipProperty) -> Tuple[bool, str]:
+def _get_relation_use_list_and_type(
+    relationship_property: RelationshipProperty,
+) -> tuple[bool, str]:
     """Get the use_list property and relationship type for a given relationship_property.
 
     Args:
@@ -615,7 +570,7 @@ def _get_relation_use_list_and_type(relationship_property: RelationshipProperty)
     return not relationship_property.uselist, direction
 
 
-def table_namer(model: Optional[Type[DeclarativeBase]] = None) -> str:
+def table_namer(model: type[DeclarativeBase] | None = None) -> str:
     """
     Get the table name from the model name by converting camelCase, PascalCase, or kebab-case to snake_case.
 
@@ -632,7 +587,7 @@ def table_namer(model: Optional[Type[DeclarativeBase]] = None) -> str:
     return convert_camel_to_snake(snake_case_name)
 
 
-def get_models_relationships(model: Type[DeclarativeBase]) -> List[Dict[str, Any]]:
+def get_models_relationships(model: type[DeclarativeBase]) -> list[dict[str, Any]]:
     """
     Get the relationships of the model, including the join key and columns.
 
@@ -654,13 +609,12 @@ def get_models_relationships(model: Type[DeclarativeBase]) -> List[Dict[str, Any
             relationships.append(relationship_info)
             logger.debug(
                 4,
-                f"Added |{model.__name__}| relationship with |{relationship_info['model'].__name__}| "
-                f"via columns {relationship_info['left_column']} to {relationship_info['right_column']}."
+                f"Added |{model.__name__}| relationship with |{relationship_info['model'].__name__}| via columns {relationship_info['left_column']} to {relationship_info['right_column']}.",
             )
     return relationships
 
 
-def extract_relationship_info(rel: RelationshipProperty) -> Dict[str, Any]:
+def extract_relationship_info(rel: RelationshipProperty) -> dict[str, Any]:
     """
     Extract detailed information from a relationship property.
 
@@ -693,7 +647,7 @@ def extract_relationship_info(rel: RelationshipProperty) -> Dict[str, Any]:
         return {}
 
 
-def get_primary_keys(model: Type[DeclarativeBase]) -> Column:
+def get_primary_keys(model: type[DeclarativeBase]) -> Column:
     """
     Get the primary key column for the model.
 
@@ -724,12 +678,13 @@ def get_primary_key_filters(base_model, lookup_val):
         return {pks[0].name: lookup_val}
 
     # If there are multiple primary key columns, split the lookup_val and map accordingly
-    if isinstance(lookup_val, (tuple, list)):
-        return {pk.name: val for pk, val in zip(pks, lookup_val)}
+    if isinstance(lookup_val, tuple | list):
+        return {pk.name: val for pk, val in zip(pks, lookup_val, strict=False)}
 
     raise ValueError(f"Multiple primary keys found in {base_model.__name__}, but lookup_val is not a tuple or list.")
 
-def list_model_columns(model: Type[DeclarativeBase]) -> List[str]:
+
+def list_model_columns(model: type[DeclarativeBase]) -> list[str]:
     """
     Get the list of columns for the model, including hybrid properties.
 
@@ -743,7 +698,7 @@ def list_model_columns(model: Type[DeclarativeBase]) -> List[str]:
     return list(all_columns.values())[0].keys()
 
 
-def _extract_model_attributes(model: Type[DeclarativeBase]) -> Tuple[set, set]:
+def _extract_model_attributes(model: type[DeclarativeBase]) -> tuple[set, set]:
     """
     Extracts column and property keys from the model.
 
@@ -811,14 +766,10 @@ def get_related_b_query(model_a, model_b, a_pk_value, session):
     if source_model == model_a:
         # Relationship is from model_a to model_b
         relationship_attr = getattr(model_a, relationship_name)
-        query = (session.query(model_b)
-                 .join(relationship_attr)
-                 .filter(pk_attr_a == a_pk_value))
+        query = session.query(model_b).join(relationship_attr).filter(pk_attr_a == a_pk_value)
     else:
         # Relationship is from model_b to model_a
         relationship_attr = getattr(model_b, relationship_name)
-        query = (session.query(model_b)
-                 .join(relationship_attr)
-                 .filter(pk_attr_a == a_pk_value))
+        query = session.query(model_b).join(relationship_attr).filter(pk_attr_a == a_pk_value)
 
     return query
