@@ -1,24 +1,29 @@
 import copy
 import os
 import random
+from collections.abc import Callable
 from copy import deepcopy
 from datetime import datetime, timedelta
-from typing import Dict, Any, Optional, List, Callable, Type
+from typing import Any
 
 import pytz
 from flask import current_app
 from marshmallow import Schema, fields
-from marshmallow.schema import SchemaMeta
-from marshmallow_sqlalchemy.fields import RelatedList, Related, Nested
+from marshmallow_sqlalchemy.fields import Nested, Related, RelatedList
 from sqlalchemy.orm import DeclarativeBase
 from werkzeug.http import HTTP_STATUS_CODES
-
 from werkzeug.routing import IntegerConverter, UnicodeConverter
 
 import flarchitect.utils
 import flarchitect.utils.decorators
-from flarchitect.database.operations import get_model_relationships, get_model_columns
+from flarchitect.database.inspections import (
+    get_model_columns,
+    get_model_relationships,
+)
+from flarchitect.database.utils import AGGREGATE_FUNCS
 from flarchitect.logging import logger
+from flarchitect.utils.config_helpers import get_config_or_model_meta
+from flarchitect.utils.core_utils import convert_case
 from flarchitect.utils.general import (
     DATE_FORMAT,
     DATETIME_FORMAT,
@@ -26,17 +31,14 @@ from flarchitect.utils.general import (
     manual_render_absolute_template,
     pluralize_last_word,
 )
-from flarchitect.utils.core_utils import convert_case
-from flarchitect.utils.config_helpers import get_config_or_model_meta
-from flarchitect.database.utils import AGGREGATE_FUNCS
 
 
 def scrape_extra_info_from_spec_data(
-    spec_data: Dict[str, Any],
+    spec_data: dict[str, Any],
     method: str,
     multiple: bool = False,
     summary: bool = False,
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     """
     Scrapes extra information from the spec data for API documentation generation.
 
@@ -58,9 +60,7 @@ def scrape_extra_info_from_spec_data(
         logger.log(1, "Missing data for documentation generation")
 
     if spec_data.get("tag") is None:
-        new_tag = get_config_or_model_meta(
-            "tag", model, output_schema, input_schema, "Unknown"
-        )
+        new_tag = get_config_or_model_meta("tag", model, output_schema, input_schema, "Unknown")
         if new_tag:
             spec_data["tag"] = new_tag
     else:
@@ -69,9 +69,7 @@ def scrape_extra_info_from_spec_data(
     if not summary and get_config_or_model_meta("AUTO_NAME_ENDPOINTS", default=True):
         schema = spec_data.get("output_schema") or spec_data.get("input_schema")
         if schema:
-            spec_data["summary"] = make_endpoint_description(
-                schema, method, **spec_data
-            )
+            spec_data["summary"] = make_endpoint_description(schema, method, **spec_data)
     else:
         spec_data["summary"] = summary
 
@@ -87,16 +85,14 @@ def scrape_extra_info_from_spec_data(
         else:
             config_val = f"{method.lower()}" + description_type
 
-        new_desc = get_config_or_model_meta(
-            config_val, model, output_schema, input_schema, None
-        )
+        new_desc = get_config_or_model_meta(config_val, model, output_schema, input_schema, None)
         if new_desc:
             spec_data[description_type] = new_desc
 
     return spec_data
 
 
-def get_summary_description(f: Callable) -> Optional[str]:
+def get_summary_description(f: Callable) -> str | None:
     """
     Retrieves and formats the summary description from a function's docstring.
 
@@ -110,7 +106,7 @@ def get_summary_description(f: Callable) -> Optional[str]:
     return description.replace("  ", " ") if description else None
 
 
-def get_param_schema(converter) -> Dict[str, str]:
+def get_param_schema(converter) -> dict[str, str]:
     """Helper function to get the schema for a parameter based on its converter.
 
     Args:
@@ -160,9 +156,7 @@ def get_param_schema(converter) -> Dict[str, str]:
 #     return query_params
 
 
-def generate_delete_query_params(
-    schema: Schema, model: DeclarativeBase
-) -> List[Dict[str, Any]]:
+def generate_delete_query_params(schema: Schema, model: DeclarativeBase) -> list[dict[str, Any]]:
     """Helper function to generate query parameters for DELETE method.
 
     Args:
@@ -174,9 +168,7 @@ def generate_delete_query_params(
     """
     query_params = []
 
-    if get_config_or_model_meta(
-        "API_ALLOW_CASCADE_DELETE", getattr(schema.Meta, "model", None), default=True
-    ):
+    if get_config_or_model_meta("API_ALLOW_CASCADE_DELETE", getattr(schema.Meta, "model", None), default=True):
         query_params.append(
             {
                 "name": "cascade_delete",
@@ -188,9 +180,7 @@ def generate_delete_query_params(
     return query_params
 
 
-def generate_get_query_params(
-    schema: Schema, model: DeclarativeBase
-) -> List[Dict[str, Any]]:
+def generate_get_query_params(schema: Schema, model: DeclarativeBase) -> list[dict[str, Any]]:
     """Helper function to generate query parameters for GET method.
 
     Args:
@@ -232,9 +222,7 @@ def generate_get_query_params(
     return query_params
 
 
-def generate_additional_query_params(
-    methods: set, schema: Schema, model: DeclarativeBase
-) -> List[Dict[str, Any]]:
+def generate_additional_query_params(methods: set, schema: Schema, model: DeclarativeBase) -> list[dict[str, Any]]:
     """Helper function to generate additional query parameters.
 
     Args:
@@ -259,10 +247,10 @@ def generate_additional_query_params(
 
 
 def _add_request_body_to_spec_template(
-    spec_template: Dict[str, Any],
+    spec_template: dict[str, Any],
     http_method: str,
     input_schema: Schema,
-    model: Optional[DeclarativeBase],
+    model: DeclarativeBase | None,
 ):
     """Helper function to add a request body to the spec template.
 
@@ -277,8 +265,7 @@ def _add_request_body_to_spec_template(
     """
     case = get_config_or_model_meta("API_SCHEMA_CASE", model=model, default="camel")
     name = convert_case(
-        ("patch_" if http_method == "PATCH" else "")
-        + input_schema.__name__.replace("Schema", ""),
+        ("patch_" if http_method == "PATCH" else "") + input_schema.__name__.replace("Schema", ""),
         case,
     )
 
@@ -293,9 +280,7 @@ def _add_request_body_to_spec_template(
     }
 
 
-def _add_response_to_spec_template(
-    spec_template: Dict[str, Any], output_schema: Schema
-):
+def _add_response_to_spec_template(spec_template: dict[str, Any], output_schema: Schema):
     """Helper function to add a response to the spec template.
 
     Args:
@@ -306,9 +291,7 @@ def _add_response_to_spec_template(
         None
     """
     model = output_schema.get_model() if hasattr(output_schema, "get_model") else None
-    case = get_config_or_model_meta(
-        "API_SCHEMA_CASE", model=model, default="camel"
-    )
+    case = get_config_or_model_meta("API_SCHEMA_CASE", model=model, default="camel")
     name = convert_case(output_schema.__name__.replace("Schema", ""), case)
 
     spec_template.setdefault("responses", {}).setdefault("200", {}).update(
@@ -323,9 +306,7 @@ def _add_response_to_spec_template(
     )
 
 
-def _initialize_base_responses(
-    method: str, many: bool, error_responses: List[int]
-) -> Dict[str, Dict[str, Any]]:
+def _initialize_base_responses(method: str, many: bool, error_responses: list[int]) -> dict[str, dict[str, Any]]:
     """Helper function to initialize base responses.
 
     Args:
@@ -341,24 +322,16 @@ def _initialize_base_responses(
     if 500 in error_responses or not error_responses:
         responses["500"] = {"description": HTTP_STATUS_CODES.get(500)}
 
-    if (
-        method != "POST"
-        and not many
-        and (404 in error_responses or not error_responses)
-    ):
+    if method != "POST" and not many and (404 in error_responses or not error_responses):
         responses["404"] = {"description": HTTP_STATUS_CODES.get(404)}
 
-    if (
-        method == "DELETE"
-        and not many
-        and (409 in error_responses or not error_responses)
-    ):
+    if method == "DELETE" and not many and (409 in error_responses or not error_responses):
         responses["409"] = {"description": HTTP_STATUS_CODES.get(409)}
 
     return responses
 
 
-def _initialize_auth_responses(error_responses: List[int]) -> Dict[str, Dict[str, Any]]:
+def _initialize_auth_responses(error_responses: list[int]) -> dict[str, dict[str, Any]]:
     """Helper function to initialize authentication responses.
 
     Args:
@@ -379,7 +352,7 @@ def _initialize_auth_responses(error_responses: List[int]) -> Dict[str, Dict[str
     return responses
 
 
-def _initialize_rate_limit_responses(rate_limit: bool) -> Dict[str, Dict[str, Any]]:
+def _initialize_rate_limit_responses(rate_limit: bool) -> dict[str, dict[str, Any]]:
     """Helper function to initialize rate limit responses.
 
     Args:
@@ -421,7 +394,7 @@ def _initialize_rate_limit_responses(rate_limit: bool) -> Dict[str, Dict[str, An
     }
 
 
-def get_template_data_for_model(schema: Schema) -> Optional[Dict[str, Any]]:
+def get_template_data_for_model(schema: Schema) -> dict[str, Any] | None:
     """Generates model data for Jinja template and Redoc.
 
     Args:
@@ -438,20 +411,16 @@ def get_template_data_for_model(schema: Schema) -> Optional[Dict[str, Any]]:
         base_fields = get_model_columns(base_model)
 
         model_relationships = get_model_relationships(base_model)
-        model_relationship_names = [
-            convert_case(x.__name__, schema_case) for x in model_relationships
-        ]
+        model_relationship_names = [convert_case(x.__name__, schema_case) for x in model_relationships]
 
         if model_relationships:
             relationship_fields = get_model_columns(model_relationships[0])
-            relationship_resource = convert_case(
-                model_relationships[0].__name__, schema_case
-            )
+            relationship_resource = convert_case(model_relationships[0].__name__, schema_case)
         else:
             relationship_fields = []
             relationship_resource = None
 
-        aggs = ", ".join([f"`{x}`" for x in AGGREGATE_FUNCS.keys()])
+        aggs = ", ".join([f"`{x}`" for x in AGGREGATE_FUNCS])
 
         return {
             "relationship_resource": relationship_resource,
@@ -464,9 +433,7 @@ def get_template_data_for_model(schema: Schema) -> Optional[Dict[str, Any]]:
     return None
 
 
-def generate_example_values(
-    now: datetime, yesterday: datetime, day_before_yesterday: datetime
-) -> Dict[str, List[str]]:
+def generate_example_values(now: datetime, yesterday: datetime, day_before_yesterday: datetime) -> dict[str, list[str]]:
     """Helper function to generate example values for filter examples.
 
     Args:
@@ -496,7 +463,7 @@ def generate_example_values(
     }
 
 
-def generate_operators() -> Dict[str, List[str]]:
+def generate_operators() -> dict[str, list[str]]:
     """Helper function to generate operators for different data types.
 
     Returns:
@@ -537,9 +504,9 @@ def generate_operators() -> Dict[str, List[str]]:
 
 def generate_operator_examples(
     schema: Schema,
-    operators: Dict[str, List[str]],
-    example_values: Dict[str, List[str]],
-) -> List[str]:
+    operators: dict[str, list[str]],
+    example_values: dict[str, list[str]],
+) -> list[str]:
     """Helper function to generate operator examples for filters.
 
     Args:
@@ -558,7 +525,6 @@ def generate_operator_examples(
     except TypeError:
         fields = schema.dump_fields
 
-
     # Generate column examples only for those fields that are not dump_only
     columns = [k for k, v in fields.items() if v and not v.dump_only]
 
@@ -567,9 +533,7 @@ def generate_operator_examples(
         if col_type in operators:
             chosen_operator = random.choice(operators[col_type])
             if chosen_operator in ["__in", "__nin"]:
-                chosen_values = ", ".join(
-                    random.choices(example_values.get(col_type, ["value"]), k=3)
-                )
+                chosen_values = ", ".join(random.choices(example_values.get(col_type, ["value"]), k=3))
                 examples.append(f"{column}{chosen_operator}=({chosen_values})")
             else:
                 chosen_value = random.choice(example_values.get(col_type, ["value"]))
@@ -578,9 +542,7 @@ def generate_operator_examples(
     return examples
 
 
-def get_table_name(
-    index: int, resource_name: str, fields: List[str], example_table: List[str]
-) -> str:
+def get_table_name(index: int, resource_name: str, fields: list[str], example_table: list[str]) -> str:
     """Helper function to generate table names for field examples.
 
     Args:
@@ -602,17 +564,13 @@ def get_table_name(
         current_fields = []
         for _ in range(5):
             table_choice = random.choice([resource_name, "OtherTable"])
-            field_choice = (
-                random.choice(temp_fields)
-                if table_choice == resource_name
-                else random.choice(example_table)
-            )
+            field_choice = random.choice(temp_fields) if table_choice == resource_name else random.choice(example_table)
             current_fields.append(f"{table_choice}.{field_choice}")
 
     return "fields=" + ",".join(current_fields)
 
 
-def _prepare_patch_schema(input_schema: Optional[Schema]) -> Optional[Schema]:
+def _prepare_patch_schema(input_schema: Schema | None) -> Schema | None:
     """Prepare a patch schema with all fields optional.
 
     Args:
@@ -642,15 +600,15 @@ def _prepare_patch_schema(input_schema: Optional[Schema]) -> Optional[Schema]:
         class_fields[field_name] = new_field
 
     # Preserve the Meta class if it exists
-    Meta = getattr(input_schema.__class__, 'Meta', None)
+    Meta = getattr(input_schema.__class__, "Meta", None)
     if Meta:
-        class_fields['Meta'] = Meta
+        class_fields["Meta"] = Meta
 
     # Dynamically create a new schema class with the modified fields
     PatchSchemaClass = type(
         f"Patch{input_schema.__class__.__name__}",
         (Schema,),  # Inherit directly from Schema to avoid field conflicts
-        class_fields
+        class_fields,
     )
 
     # Instantiate the new schema class
@@ -658,6 +616,7 @@ def _prepare_patch_schema(input_schema: Optional[Schema]) -> Optional[Schema]:
 
     return put_input_schema
 
+
 def make_endpoint_description(schema: Schema, http_method: str, **kwargs) -> str:
     """Generates endpoint description from a schema for the API docs.
 
@@ -673,9 +632,7 @@ def make_endpoint_description(schema: Schema, http_method: str, **kwargs) -> str
     many = kwargs.get("multiple")
     model = getattr(schema, "get_model", lambda: None)()
     name = (kwargs.get("model") or model or schema).__name__.replace("Schema", "")
-    name = convert_case(
-        name, get_config_or_model_meta("API_SCHEMA_CASE", model=model, default="camel")
-    )
+    name = convert_case(name, get_config_or_model_meta("API_SCHEMA_CASE", model=model, default="camel"))
 
     parent = kwargs.get("parent")
     parent_name = parent.__name__ if parent else ""
@@ -712,20 +669,17 @@ def generate_fields_description(schema: Schema) -> str:
     if callable(schema):
         schema = schema()
 
-    fields = [
-        (k, v.metadata.get("description", ""))
-        for k, v in schema.fields.items()
-        if v and v.dump_only is False and not isinstance(v, (RelatedList, Related))
-    ]
+    fields = [(k, v.metadata.get("description", "")) for k, v in schema.fields.items() if v and v.dump_only is False and not isinstance(v, RelatedList | Related)]
 
     if hasattr(schema, "Meta") and hasattr(schema.Meta, "model"):
         resource_name = schema.Meta.model.__name__
-        example_table = (
-            "OtherTable.name,OtherTable.age,OtherTable.id,OtherTable.email".split(",")
-        )
-        example_fields = [
-            get_table_name(i, resource_name, fields, example_table) for i in range(3)
+        example_table = [
+            "OtherTable.name",
+            "OtherTable.age",
+            "OtherTable.id",
+            "OtherTable.email",
         ]
+        example_fields = [get_table_name(i, resource_name, fields, example_table) for i in range(3)]
 
         full_path = os.path.join(html_path, "redoc_templates/fields.html")
         schema_name = endpoint_namer(schema.Meta.model)
@@ -742,7 +696,7 @@ def generate_fields_description(schema: Schema) -> str:
     return "None"
 
 
-def generate_x_description(template_data: Dict, path: str = "") -> str:
+def generate_x_description(template_data: dict, path: str = "") -> str:
     """Generates filter examples from a model.
 
     Args:
@@ -783,9 +737,7 @@ def generate_filter_examples(schema: Schema) -> str:
 
     full_path = os.path.join(html_path, "redoc_templates/filters.html")
 
-    return manual_render_absolute_template(
-        full_path, examples=[example_one, example_two]
-    )
+    return manual_render_absolute_template(full_path, examples=[example_one, example_two])
 
 
 def convert_path_to_openapi(path: str) -> str:
@@ -804,8 +756,8 @@ def initialize_spec_template(
     method: str,
     many: bool = False,
     rate_limit: bool = False,
-    error_responses: Optional[List[int]] = None,
-) -> Dict[str, Any]:
+    error_responses: list[int] | None = None,
+) -> dict[str, Any]:
     """Initializes the spec template with optional rate limiting headers for successful and rate-limited responses.
 
     Args:
@@ -828,13 +780,13 @@ def initialize_spec_template(
 
 
 def append_parameters(
-    spec_template: Dict[str, Any],
-    query_params: List[Dict[str, Any]],
-    path_params: List[Dict[str, Any]],
+    spec_template: dict[str, Any],
+    query_params: list[dict[str, Any]],
+    path_params: list[dict[str, Any]],
     http_method: str,
-    input_schema: Optional[Schema] = None,
-    output_schema: Optional[Schema] = None,
-    model: Optional[DeclarativeBase] = None,
+    input_schema: Schema | None = None,
+    output_schema: Schema | None = None,
+    model: DeclarativeBase | None = None,
     many: bool = False,
 ):
     """Enhances a spec template with parameters, request bodies, and responses for API documentation.
@@ -852,9 +804,7 @@ def append_parameters(
     Returns:
         None
     """
-    flarchitect.utils.general.html_path = current_app.extensions[
-        "flarchitect"
-    ].get_templates_path()
+    flarchitect.utils.general.html_path = current_app.extensions["flarchitect"].get_templates_path()
 
     spec_template.setdefault("parameters", []).extend(path_params + query_params)
     rate_limit = get_config_or_model_meta(
@@ -867,24 +817,15 @@ def append_parameters(
 
     if rate_limit:
         description = spec_template.get("description", "")
-        spec_template["description"] = (
-            f"{description}\n**Rate Limited** - requests on this endpoint are limited to `{rate_limit}`."
-        )
+        spec_template["description"] = f"{description}\n**Rate Limited** - requests on this endpoint are limited to `{rate_limit}`."
 
     if input_schema:
-        _add_request_body_to_spec_template(
-            spec_template, http_method, input_schema, model
-        )
+        _add_request_body_to_spec_template(spec_template, http_method, input_schema, model)
 
     if output_schema:
         _add_response_to_spec_template(spec_template, output_schema)
 
-    if (
-        http_method == "GET"
-        and model
-        and many
-        and get_config_or_model_meta("API_ALLOW_FILTERS", model=model, default=True)
-    ):
+    if http_method == "GET" and model and many and get_config_or_model_meta("API_ALLOW_FILTERS", model=model, default=True):
         spec_template["parameters"].append(
             {
                 "name": "filters",
@@ -895,14 +836,12 @@ def append_parameters(
         )
 
         template_data = get_template_data_for_model(output_schema)
-        spec_template["parameters"].extend(
-            make_endpoint_params_description(output_schema, template_data)
-        )
+        spec_template["parameters"].extend(make_endpoint_params_description(output_schema, template_data))
 
     add_auth_to_spec(model, spec_template)
 
 
-def add_auth_to_spec(model: DeclarativeBase, spec_template: Dict[str, Any]):
+def add_auth_to_spec(model: DeclarativeBase, spec_template: dict[str, Any]):
     """Adds authentication information to the spec template.
 
     Args:
@@ -913,9 +852,7 @@ def add_auth_to_spec(model: DeclarativeBase, spec_template: Dict[str, Any]):
         None
     """
     auth_on = get_config_or_model_meta("API_AUTHENTICATE", model=model, default=False)
-    auth_type = get_config_or_model_meta(
-        "API_AUTHENTICATE_METHOD", model=model, default=None
-    )
+    auth_type = get_config_or_model_meta("API_AUTHENTICATE_METHOD", model=model, default=None)
 
     if not auth_on:
         return
@@ -950,9 +887,7 @@ def add_auth_to_spec(model: DeclarativeBase, spec_template: Dict[str, Any]):
         }
 
 
-def make_endpoint_params_description(
-    schema: Schema, data: Dict[str, Any]
-) -> List[Dict[str, Any]]:
+def make_endpoint_params_description(schema: Schema, data: dict[str, Any]) -> list[dict[str, Any]]:
     """Generates endpoint parameters description from a schema for the API docs.
 
     Args:
@@ -964,9 +899,7 @@ def make_endpoint_params_description(
     """
     params = []
 
-    if get_config_or_model_meta(
-        "API_ALLOW_SELECT_FIELDS", getattr(schema.Meta, "model", None), default=True
-    ):
+    if get_config_or_model_meta("API_ALLOW_SELECT_FIELDS", getattr(schema.Meta, "model", None), default=True):
         params.append(
             {
                 "name": "fields",
@@ -976,66 +909,50 @@ def make_endpoint_params_description(
             }
         )
 
-    if get_config_or_model_meta(
-        "API_ALLOW_ORDER_BY", getattr(schema.Meta, "model", None), default=True
-    ):
+    if get_config_or_model_meta("API_ALLOW_ORDER_BY", getattr(schema.Meta, "model", None), default=True):
         params.append(
             {
                 "name": "order by",
                 "in": "query",
                 "schema": {"type": "string"},
-                "description": generate_x_description(
-                    data, "redoc_templates/order.html"
-                ),
+                "description": generate_x_description(data, "redoc_templates/order.html"),
             }
         )
 
-    if get_config_or_model_meta(
-        "API_ALLOW_JOIN", getattr(schema.Meta, "model", None), default=False
-    ):
+    if get_config_or_model_meta("API_ALLOW_JOIN", getattr(schema.Meta, "model", None), default=False):
         params.append(
             {
-                "name": "joins",
+                "name": "join",
                 "in": "query",
                 "schema": {"type": "string"},
-                "description": generate_x_description(
-                    data, "redoc_templates/joins.html"
-                ),
+                "description": generate_x_description(data, "redoc_templates/joins.html"),
             }
         )
 
-    if get_config_or_model_meta(
-        "API_ALLOW_GROUPBY", getattr(schema.Meta, "model", None), default=False
-    ):
+    if get_config_or_model_meta("API_ALLOW_GROUPBY", getattr(schema.Meta, "model", None), default=False):
         params.append(
             {
-                "name": "group by",
+                "name": "groupby",
                 "in": "query",
                 "schema": {"type": "string"},
-                "description": generate_x_description(
-                    data, "redoc_templates/group.html"
-                ),
+                "description": generate_x_description(data, "redoc_templates/group.html"),
             }
         )
 
-    if get_config_or_model_meta(
-        "API_ALLOW_AGGREGATION", getattr(schema.Meta, "model", None), default=False
-    ):
+    if get_config_or_model_meta("API_ALLOW_AGGREGATION", getattr(schema.Meta, "model", None), default=False):
         params.append(
             {
                 "name": "aggregation",
                 "in": "query",
                 "schema": {"type": "string"},
-                "description": generate_x_description(
-                    data, "redoc_templates/aggregate.html"
-                ),
+                "description": generate_x_description(data, "redoc_templates/aggregate.html"),
             }
         )
 
     return params
 
 
-def handle_authorization(f: Callable, spec_template: Dict[str, Any]):
+def handle_authorization(f: Callable, spec_template: dict[str, Any]):
     """Handles authorization in the documentation.
 
     Args:
@@ -1045,12 +962,12 @@ def handle_authorization(f: Callable, spec_template: Dict[str, Any]):
     Returns:
         None
     """
-    roles_required = None
+    required_roles = None
 
     if hasattr(f, "_decorators"):
         for decorator in f._decorators:
-            if decorator.__name__ == "auth_required":
-                roles_required = decorator._args
+            if decorator.__name__ == "roles_required":
+                required_roles = decorator._args
                 spec_template["parameters"].append(
                     {
                         "name": "Authorization",
@@ -1062,14 +979,12 @@ def handle_authorization(f: Callable, spec_template: Dict[str, Any]):
                 )
                 break
 
-    if roles_required:
-        roles_desc = ", ".join(roles_required)
-        spec_template["responses"]["401"][
-            "description"
-        ] += f" Roles required: {roles_desc}."
+    if required_roles:
+        roles_desc = ", ".join(required_roles)
+        spec_template["responses"]["401"]["description"] += f" Roles required: {roles_desc}."
 
 
-def get_openapi_meta_data(field_obj: fields.Field) -> Dict[str, Any]:
+def get_openapi_meta_data(field_obj: fields.Field) -> dict[str, Any]:
     """Convert a Marshmallow field to an OpenAPI type.
 
     Args:
@@ -1081,23 +996,13 @@ def get_openapi_meta_data(field_obj: fields.Field) -> Dict[str, Any]:
     openapi_type_info = {}
     field_type = type(field_obj)
 
-    if (
-        hasattr(field_obj, "parent")
-        and hasattr(field_obj.parent, "Meta")
-        and hasattr(field_obj.parent.Meta, "model")
-    ):
-        openapi_type_info = get_description_and_example_add(
-            openapi_type_info, field_obj
-        )
+    if hasattr(field_obj, "parent") and hasattr(field_obj.parent, "Meta") and hasattr(field_obj.parent.Meta, "model"):
+        openapi_type_info = get_description_and_example_add(openapi_type_info, field_obj)
 
     openapi_type_info["type"] = type_mapping.get(field_type, "string")
 
     if field_type in [fields.DateTime, fields.Date, fields.Time]:
-        openapi_type_info["format"] = (
-            "date-time"
-            if field_type == fields.DateTime
-            else field_type.__name__.lower()
-        )
+        openapi_type_info["format"] = "date-time" if field_type == fields.DateTime else field_type.__name__.lower()
 
     if field_type == fields.Function:
         openapi_type_info["format"] = "uri"
@@ -1109,14 +1014,12 @@ def get_openapi_meta_data(field_obj: fields.Field) -> Dict[str, Any]:
     if field_type in [Nested, Related, RelatedList]:
         related_schema_name = get_related_schema_name(field_obj, field_type)
         if related_schema_name:
-            openapi_type_info = handle_nested_related_fields(
-                field_obj, field_type, related_schema_name, openapi_type_info
-            )
+            openapi_type_info = handle_nested_related_fields(field_obj, field_type, related_schema_name, openapi_type_info)
 
     return openapi_type_info
 
 
-def get_related_schema_name(field_obj: fields.Field, field_type: type) -> Optional[str]:
+def get_related_schema_name(field_obj: fields.Field, field_type: type) -> str | None:
     """Get the related schema name for nested or related fields.
 
     Args:
@@ -1139,8 +1042,8 @@ def handle_nested_related_fields(
     field_obj: fields.Field,
     field_type: type,
     related_schema_name: str,
-    openapi_type_info: Dict[str, Any],
-) -> Dict[str, Any]:
+    openapi_type_info: dict[str, Any],
+) -> dict[str, Any]:
     """Handle the OpenAPI metadata for nested or related fields.
 
     Args:
@@ -1154,17 +1057,13 @@ def handle_nested_related_fields(
     """
     if field_obj.many or field_type == RelatedList:
         openapi_type_info["type"] = "array"
-        openapi_type_info["items"] = {
-            "$ref": f"#/components/schemas/{related_schema_name}"
-        }
+        openapi_type_info["items"] = {"$ref": f"#/components/schemas/{related_schema_name}"}
     else:
         openapi_type_info["$ref"] = f"#/components/schemas/{related_schema_name}"
     return openapi_type_info
 
 
-def get_description_and_example_add(
-    openapi_type_info: Dict[str, Any], field_obj: fields.Field
-) -> Dict[str, Any]:
+def get_description_and_example_add(openapi_type_info: dict[str, Any], field_obj: fields.Field) -> dict[str, Any]:
     """Add description and example to the OpenAPI metadata from the model field.
 
     Args:
@@ -1204,7 +1103,7 @@ type_mapping = {
 }
 
 
-def get_description(kwargs: Dict[str, Any]) -> str:
+def get_description(kwargs: dict[str, Any]) -> str:
     """
     Get the description for the route from the model's Meta class or a default description based on the method.
 
@@ -1221,11 +1120,7 @@ def get_description(kwargs: Dict[str, Any]) -> str:
         parent = kwargs["parent_model"]
         return f"Get multiple `{name}` records from the database based on the parent {endpoint_namer(parent)} id"
 
-    description = (
-        getattr(model.Meta, "description", {}).get(method)
-        if hasattr(model, "Meta")
-        else None
-    )
+    description = getattr(model.Meta, "description", {}).get(method) if hasattr(model, "Meta") else None
     if description:
         return description
 
@@ -1238,7 +1133,7 @@ def get_description(kwargs: Dict[str, Any]) -> str:
     return default_descriptions.get(method, "")
 
 
-def get_tag_group(kwargs: Dict[str, Any]) -> Optional[str]:
+def get_tag_group(kwargs: dict[str, Any]) -> str | None:
     """
     Get the x-tagGroup for the route from the model's Meta class.
 
@@ -1253,9 +1148,9 @@ def get_tag_group(kwargs: Dict[str, Any]) -> Optional[str]:
 
 
 def endpoint_namer(
-    model: Optional[Type[DeclarativeBase]] = None,
-    input_schema: Optional[Type[Schema]] = None,
-    output_schema: Optional[Type[Schema]] = None,
+    model: type[DeclarativeBase] | None = None,
+    input_schema: type[Schema] | None = None,
+    output_schema: type[Schema] | None = None,
 ) -> str:
     """
     Get the endpoint name for the model based on its name.
