@@ -1,9 +1,9 @@
-from typing import Dict, Any
+from typing import Any
 
 from flarchitect.utils.config_helpers import get_config_or_model_meta
 
 
-def _filter_response_data(data: Dict[str, Any]) -> Dict[str, Any]:
+def _filter_response_data(data: dict[str, Any]) -> dict[str, Any]:
     """
     Filter response data based on configuration settings.
 
