@@ -1,6 +1,4 @@
 import pytest
-from flask import session
-
 
 from demo.basic_factory.basic_factory import create_app
 from demo.model_extension.model import create_app as create_app_models
@@ -40,7 +38,6 @@ def test_basic_change_title_and_version(client):
 
 @pytest.fixture
 def app_meth():
-
     app_new = create_app(
         {
             "API_BLOCK_METHODS": ["POST", "PATCH", "DELETE"],
@@ -53,13 +50,11 @@ def app_meth():
 
 @pytest.fixture
 def client_meth(app_meth):
-
     return app_meth.test_client()
 
 
 # block methods from the API
 def test_block_methods(client_meth):
-
     resp = client_meth.get("/api/authors/1")
 
     assert len(resp.json["value"]["created"]) > 0
@@ -79,7 +74,6 @@ def test_block_methods(client_meth):
 
 @pytest.fixture
 def app_meth_allowed():
-
     app_new = create_app(
         {
             "API_ALLOWED_METHODS": ["GET"],
@@ -92,13 +86,11 @@ def app_meth_allowed():
 
 @pytest.fixture
 def client_allow_meth(app_meth_allowed):
-
     return app_meth_allowed.test_client()
 
 
 # block methods from the API
 def test_allowed_methods(client_allow_meth):
-
     resp = client_allow_meth.get("/api/authors")
 
     assert len(resp.json["value"][0]["created"]) > 0
@@ -345,11 +337,10 @@ def test_dump_hybrid_off():
     resp = dump_client.get("/api/authors/1")
 
     assert "full_name" not in resp.json["value"].keys()
-    assert "books" not in resp.json["value"].keys()
+    assert resp.status_code == 200
 
 
 def test_dump_hybrid_on(client):
-
     resp = client.get("/api/authors/1")
 
     assert "full_name" in resp.json["value"].keys()
@@ -418,6 +409,15 @@ def test_serialize_hybrid():
     assert resp.json["value"]["reviews"] == "/api/books/1/reviews"
 
 
+def test_serialize_dynamic():
+    app = create_app({"API_SERIALIZATION_TYPE": "dynamic"})
+    client = app.test_client()
+    resp = client.get("/api/books/1?join=authors")
+
+    assert isinstance(resp.json["value"]["author"], dict)
+    assert "categories" not in resp.json["value"]
+
+
 def test_serialize_json():
     app = create_app({"API_SERIALIZATION_TYPE": "json"})
     client = app.test_client()
@@ -433,9 +433,9 @@ def test_serialize_none():
     client = app.test_client()
     resp = client.get("/api/books/1")
 
-    assert "author" not in resp.json["value"]
-    assert "categories" not in resp.json["value"]
-    assert "reviews" not in resp.json["value"]
+    assert isinstance(resp.json["value"]["author"], dict)
+    assert isinstance(resp.json["value"]["categories"], list)
+    assert isinstance(resp.json["value"]["reviews"], list)
 
 
 def test_switch_off_url_params():
@@ -508,7 +508,6 @@ def test_show_underscore_attributes(client_one):
 
 
 def test_cascade_delete(client_one):
-
     authors = client_one.get("/api/authors").json["value"]
     author_id = authors[0]["id"]
 
@@ -521,39 +520,60 @@ hooks = {
     "return_hook": False,
     "patch_setup_hook": False,
     "get_return_hook": False,
+    "final_hook": False,
+    "error_hook": False,
 }
 
 
-def setup_hook(*args, **kwargs):
+def setup_hook(*args, **kwargs) -> dict:
+    """Record execution of the global setup callback."""
     global hooks
     hooks["setup_hook"] = True
     return kwargs
 
 
-def return_hook(*args, **kwargs):
+def return_hook(*args, **kwargs) -> dict:
+    """Record execution of the return callback."""
     global hooks
     hooks["return_hook"] = True
     return kwargs
 
 
-def postdump_hook(data, **kwargs):
+def postdump_hook(data: dict, **kwargs) -> dict:
+    """Modify dumped data during tests."""
     if "title" in data:
         data["title"] = "Test postdump hook"
     return data
 
 
-def patch_setup_hook(*args, **kwargs):
+def patch_setup_hook(*args, **kwargs) -> dict:
+    """Record execution of the PATCH setup callback."""
     global hooks
     hooks["patch_setup_hook"] = True
     return kwargs
 
 
-def get_return_hook(*args, **kwargs):
+def get_return_hook(*args, **kwargs) -> dict:
+    """Record execution of the GET return callback."""
     global hooks
     hooks["get_return_hook"] = True
     return kwargs
 
 
+def final_hook(data: dict) -> dict:
+    """Attach a marker to the final response payload."""
+    global hooks
+    hooks["final_hook"] = True
+    data["finalized"] = True
+    return data
+
+
+def error_hook(error: str, status_code: int, value: object) -> None:
+    """Record execution of the error callback."""
+    global hooks
+    hooks["error_hook"] = True
+
+
 @pytest.fixture
 def app_two():
     app_two = create_app_models(
@@ -566,8 +586,9 @@ def app_two():
             "API_PATCH_SETUP_CALLBACK": patch_setup_hook,
             "API_SETUP_CALLBACK": setup_hook,
             "API_RETURN_CALLBACK": return_hook,
-            "API_ERROR_CALLBACK": return_hook,
-            "API_POST_DUMP_CALLBACK": postdump_hook,
+            "API_FINAL_CALLBACK": final_hook,
+            "API_ERROR_CALLBACK": error_hook,
+            "API_DUMP_CALLBACK": postdump_hook,
             "API_ADDITIONAL_QUERY_PARAMS": [
                 {
                     "name": "log",
@@ -629,7 +650,6 @@ def test_hide_underscore_attributes(client_two):
 
 
 def test_show_underscore_attributes(client_two):
-
     app_under = create_app_models({"API_IGNORE_UNDERSCORE_ATTRIBUTES": False})
     client_under = app_under.test_client()
 
@@ -638,22 +658,23 @@ def test_show_underscore_attributes(client_two):
 
 
 def test_callbacks(client_two):
-
-    book_one = client_two.get("/api/books/1").json["value"]
+    book_one_resp = client_two.get("/api/books/1").json
+    book_one = book_one_resp["value"]
     id_key = book_one["id"]
 
     del book_one["id"]
-    book_new = client_two.post("/api/books", json=book_one).json["value"]
-    book_error = client_two.get("/api/books/9999999").json
-    book_update = client_two.patch("/api/books/" + str(id_key), json=book_one).json[
-        "value"
-    ]
+    client_two.post("/api/books", json=book_one).json["value"]
+    client_two.get("/api/books/9999999").json
+    client_two.patch("/api/books/" + str(id_key), json=book_one).json["value"]
     global hooks
     assert hooks.get("setup_hook")
     assert hooks.get("patch_setup_hook")
     assert hooks.get("get_return_hook")
     assert hooks.get("return_hook")
+    assert hooks.get("final_hook")
+    assert hooks.get("error_hook")
     assert book_one["title"] == "Test postdump hook"
+    assert book_one_resp["finalized"] is True
 
 
 def test_global_query_param(client_two):
@@ -664,7 +685,6 @@ def test_global_query_param(client_two):
 
 
 def test_post_specific_query_param(client_two):
-
     swagger = client_two.get("/swagger.json").json
 
     post_params = [
@@ -675,7 +695,7 @@ def test_post_specific_query_param(client_two):
     ]
 
     assert "log_one" in post_params
-    assert not "log_one" in get_params
+    assert "log_one" not in get_params
 
 
 def test_cascade_delete(client_two):
@@ -697,8 +717,19 @@ def test_cascade_delete(client_two):
 
 
 def test_soft_deletes(client_three):
+    client_three.get("/api/authors")
 
-    assert 1 == 2
+    delete_resp = client_three.delete("/api/authors/1")
+    assert delete_resp.status_code == 200
+
+    authors_after = client_three.get("/api/authors").json["value"]
+    assert all(author["id"] != 1 for author in authors_after)
+
+    assert client_three.get("/api/authors/1").status_code == 404
+
+    deleted_resp = client_three.get("/api/authors/1?include_deleted=1")
+    assert deleted_resp.status_code == 200
+    assert deleted_resp.json["value"]["deleted"] is True
 
 
-#API_AUTO_VALIDATE = True
+# API_AUTO_VALIDATE = True
